#include <iostream>
#include <sstream>
#include <iterator>
#include <queue>
#include <vector>
#include <utility>
#include "bot.h"

using namespace std;
using namespace __gnu_cxx;


Bot::Bot(const string& name, PlayerType type){
	_name = name;
	_type = type;

	_playerTypeNames[robber] = "robber";
	_playerTypeNames[cop_foot] = "cop-foot";
	_playerTypeNames[cop_car] = "cop-car";

	_playerTypes["robber"] = robber;
	_playerTypes["cop-foot"] = cop_foot;
	_playerTypes["cop-car"] = cop_car;
	//shortestPath("",robber,SimpleSPGoal(""));
	//shortestPath("",robber,FindPlayer(hash_map<string,Player>(),robber));

}

void Bot::play(){
	cout << "reg: " << _name << " " << _playerTypeNames[_type] << endl;

	string input;
	getline(cin, input);
	if (input != "wsk\\")
		return;
	getline(cin, input);
	_name = value<string>(input);
	//cerr << "Got name: " << _name << endl;

	getPlayers();

	//cerr << "Got players, building graph." << endl;
	getline(cin, input);
	buildGraph();

	preGamePreparations();

	while (true){
		getline(cin, input);
		if (input == "game-over")
			return;
		//cerr << input << endl;
		//cerr << "Updating world" << endl;
		updateWorld();
		//cerr << "Done updating world" << endl;
		_type = _players[_name].type;
		_location = _players[_name].location;
		//cerr << "New turn" << endl;
		//
		string input;

		sendInformation();
		//cerr << "Getting information " << endl;
		getInformation();

		sendPlan();
		getPlans();

		vote();
		voteResult();

		move(turn());
		//cerr << "Done with turn." << endl;
	}
}

void Bot::getPlayers(){
	string input;
	//robber and 5 cops
	getline(cin, input);
	_players[value<string>(input)].type = robber;
	getline(cin, input);
	_players[value<string>(input)].type = cop_foot;
	getline(cin, input);
	_players[value<string>(input)].type = cop_foot;
	getline(cin, input);
	_players[value<string>(input)].type = cop_foot;
	getline(cin, input);
	_players[value<string>(input)].type = cop_foot;
}

/**
 	  	 	  	nod\ eol	
 	  	 	  	( nod: loc node-tag coordinate coordinate eol )*	
 	  	 	  	nod/ eol	
 	  	 	  	edg\ eol	
 	  	 	  	( edg: loc loc edge-type eol )*	
 	  	 	  	edg/ eol	
*/
void Bot::buildGraph(){
	string input;
	getline(cin, input);
	if (input != "nod\\")
		return;

	//cerr << "Getting intersections" << endl;
	while (true){
		getline(cin, input);
		if (input == "nod/")
			break;
		istringstream node(input);
		node >> input;
		string name;
		node >> name;
		Intersection& inter = _intersections[name];
		node >> input;
		if (input == "bank"){
			inter.type = bank;
			_banks[name] = 0;
		}
		else if (input == "hq"){
			inter.type = hq;
			_copHq = name;
		}
		else if (input == "robber-start"){
			inter.type = robber_start;
			_robberLocation = name;
		}
		else
			inter.type = ordinary;
		node >> inter.x;
		node >> inter.y;
	}

	//cerr << "Number of intersections: " << _intersections.size() << endl;

	getline(cin, input);
	if (input != "edg\\")
		return;

	//cerr << "Getting streets" << endl;
	int streets = 0;
	while (true){
		getline(cin, input);
		if (input == "edg/")
			break;
		++streets;
		istringstream street(input);
		string from;
		street >> from;
		street >> from;
		string to;
		street >> to;
		string type;
		street >> type;
		//cerr << "Street between: " << from << " and " << to << " of type: " << type << endl;
		if (type == "foot"){
			_intersections[from].connections[to] = both;
			Intersection& inter = _intersections[to];
			hash_map<string,StreetType>::iterator conn = inter.connections.find(from);
			if (conn == inter.connections.end())
				inter.connections[from] = foot;
			else if (conn->second == car)
				inter.connections[from] = both;
		}else{
			Intersection& inter = _intersections[from];
			hash_map<string,StreetType>::iterator conn = inter.connections.find(to);
			if (conn == inter.connections.end())
				inter.connections[from] = car;
			else if (conn->second == foot)
				inter.connections[from] = both;
		}
	}
	//cerr << "Number of streets: " << streets << endl;
	getline(cin, input);
}

void Bot::updateWorld(){
	string input;
	getline(cin,input);
	_world = value<int>(input);
	//cerr << "World: " << _world << endl;

	getline(cin,input);
	_robbed = value<int>(input);
	//cerr << "Robbed: " << _robbed << endl;
	//
	getline(cin,input);
	while (true){
		getline(cin, input);
		if (input == "dc/")
			break;
		istringstream dirtyCop(input);
	}

	getline(cin,input);
	while (true){
		getline(cin, input);
		if (input == "sc/")
			break;
		istringstream controlledCop(input);
	}

	getline(cin,input);
	while (true){
		getline(cin, input);
		if (input == "fac/")
			break;
		istringstream falseAccusation(input);
	}

	getline(cin,input);
	while (true){
		getline(cin, input);
		if (input == "bv/")
			break;
		istringstream bank(input);
		bank >> input;
		bank >> input;
		bank >> _banks[input];
	}
	//cerr << "Number of banks: " << _banks.size() << endl;
	
	getline(cin,input);
	while (true){
		getline(cin, input);
		if (input == "ev/")
			break;
		istringstream evidence(input);
		int world;
		evidence >> input;
		evidence >> input;
		evidence >> world;
		_evidence[world] = input;
	}
	
	getline(cin,input);
	_smell = value<int>(input);

	_robberLocation = "";
	getline(cin,input);
	while (true){
		getline(cin, input);
		if (input == "pl/")
			break;
		istringstream player(input);
		player >> input;
		player >> input;
		//cerr << "Player: " << input << endl;
		Player& pl = _players[input];
		player >> pl.location;
		player >> input;
		pl.type = _playerTypes[input];
		if (pl.type == robber)
			_robberLocation = pl.location;
	}
	//cerr << "Number of players: " << _players.size() << endl;
	getline(cin, input);
}

void Bot::move(std::string location){
	//cerr << "Moving to: " << location << endl;
	cout << "mov: " << location << " " << _playerTypeNames[_type] << " " << _name << endl;
}

template<class T>
T value(std::string input){
	istringstream istr(input);
	istr >> input;
	T s;
	istr >> s;
	return s;
}

std::string Bot::turn(){
	//cerr << "Using stupid stand still Bot::turn" << endl;
	return _location;
}


void Bot::sendInformation(){
	cout << "inf\\" << endl;
	cout << "inf/" << endl;
}

void Bot::getInformation(){
	string input;
	do{
		getline(cin,input);
	}while(input != "from/");
}

void Bot::sendPlan(){
	cout << "plan\\" << endl;
	cout << "plan/" << endl;
}

void Bot::getPlans(){
	string input;
	//ignore From-plan
	do{
		getline(cin,input);
	}while(input != "from/");
}

void Bot::vote(){
	cout << "vote\\" << endl;
	cout << "vote: " << _name << endl;
	priority_queue<pair<int,string> > players;
	for (hash_map<string,int>::const_iterator player = _winningPlans.begin();
			player != _winningPlans.end(); ++player){
		players.push(pair<int,string>(-player->second, player->first));
	}
	while (players.size() > 0){
		const pair<int,string>& player = players.top();
		cout << "vote: " << player.second << endl;
		//cerr << "voted for " << player.second << " with " << player.first << " previously won plans" << endl;
		players.pop();
	}
	cout << "vote/" << endl;
}

void Bot::voteResult(){
	string input;
	getline(cin,input);
	if (input != "nowinner:"){
		string winner = value<string>(input);
		if (winner != _name)
			++_winningPlans[winner];
	}
}

#include "shortestPath.cpp"
