#ifndef __BOT_H__
#define __BOT_H__

#include <ext/hash_map>
#include <list>
#include <string>
#include <locale>
#include <map>
// These are needed to be able to use std::string as key in a hash_map.
namespace __gnu_cxx {
	template< typename CharT, typename Traits, typename Alloc >
		struct hash< std::basic_string<CharT, Traits, Alloc> > {
			size_t operator()(const std::basic_string<CharT, Traits, Alloc>& s) const {

				const std::collate<CharT>& c = std::use_facet< std::collate<CharT> >(std::locale());

				return c.hash(s.c_str(), s.c_str() + s.size());

			}

		};

	template< typename CharT, typename Traits, typename Alloc >
		struct hash< const std::basic_string<CharT, Traits, Alloc> > { //yes you need this version aswell!

			size_t operator()(const std::basic_string<CharT, Traits, Alloc>& s) const {

				const std::collate<CharT>& c = std::use_facet< std::collate<CharT> >(std::locale());

				return c.hash(s.c_str(), s.c_str() + s.size());
			}

		};
};

enum StreetType{foot, car, both};
enum PlayerType{robber, cop_foot, cop_car};
enum IntersectionType{hq, bank, robber_start, ordinary};

struct Intersection{
	__gnu_cxx::hash_map<std::string,StreetType> connections;
	IntersectionType type;
	int x;
	int y;
};

struct Player{
	PlayerType type;
	std::string location;
};

struct Bank{
	std::string location;
	int value;
};

template<class T>
T value(std::string input);

struct SPInfo{
	std::string name;
	bool settled;
	SPInfo* parent;
	int cost;
};

class Bot {
	public:
		Bot(const std::string& name, PlayerType type);
		virtual ~Bot(){};

		virtual void play();

	protected:
		void buildGraph();
		void updateWorld();
		virtual std::string turn();
		virtual void preGamePreparations(){};
		virtual void move(std::string location);
		virtual void sendInformation();
		virtual void getInformation();
		virtual void sendPlan();
		virtual void getPlans();
		virtual void vote();
		virtual void voteResult();
		void getPlayers();
		template<class Goal>
		std::list<std::string> shortestPath(const std::string& from, PlayerType type, const Goal& goal, bool reverse = false);

		__gnu_cxx::hash_map<std::string, Intersection> _intersections;
		__gnu_cxx::hash_map<std::string, Player> _players;
		__gnu_cxx::hash_map<std::string, int> _banks;
		std::map<int, std::string> _evidence;
		std::map<PlayerType, std::string> _playerTypeNames;
		std::map<std::string, PlayerType> _playerTypes;
		std::string _name;
		PlayerType _type;
		std::string _location;
		int _world;
		int _robbed;
		int _smell;
		std::string _robberLocation;
		std::string _copHq;
		__gnu_cxx::hash_map<std::string, int> _winningPlans;
};

class SimpleSPGoal{
	std::string _to;
	int _limit;
	public:
	SimpleSPGoal(const std::string& to, int limit = 0):_to(to), _limit(limit){
	}
	inline int operator()(const SPInfo* node) const{
		if (_limit > 0 && node->cost >= _limit)
			return -1;
		if (node->name == _to)
			return 1;
		return 0;
	}

};

class FindPlayer{
	int _limit;
	const __gnu_cxx::hash_map<std::string, Player>& _players;
	PlayerType _type;
	public:
	FindPlayer(const __gnu_cxx::hash_map<std::string, Player>& players, PlayerType type, int limit = 0): _players(players), _type(type), _limit(limit){
	}
	inline int operator()(const SPInfo* node) const{
		if (_limit > 0 && node->cost >= _limit)
			return -1;
		for(__gnu_cxx::hash_map<std::string, Player>::const_iterator player = _players.begin();
				player != _players.end(); ++player){
			if (player->second.type == _type && player->second.location == node->name){
				return 1;
			}
		}
		return 0;
	}

};

struct SPInfoComp{
	inline bool operator()(const SPInfo* v1, const SPInfo* v2){
		return v1->cost > v2->cost;
	}
};


#endif
